# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import portability_check_run_params
from .._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.portability_check_run_response import PortabilityCheckRunResponse

__all__ = ["PortabilityChecksResource", "AsyncPortabilityChecksResource"]


class PortabilityChecksResource(SyncAPIResource):
    """Determining portability of phone numbers"""

    @cached_property
    def with_raw_response(self) -> PortabilityChecksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return PortabilityChecksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PortabilityChecksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return PortabilityChecksResourceWithStreamingResponse(self)

    def run(
        self,
        *,
        phone_numbers: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PortabilityCheckRunResponse:
        """
        Runs a portability check, returning the results immediately.

        Args:
          phone_numbers: The list of +E.164 formatted phone numbers to check for portability

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/portability_checks",
            body=maybe_transform(
                {"phone_numbers": phone_numbers}, portability_check_run_params.PortabilityCheckRunParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PortabilityCheckRunResponse,
        )


class AsyncPortabilityChecksResource(AsyncAPIResource):
    """Determining portability of phone numbers"""

    @cached_property
    def with_raw_response(self) -> AsyncPortabilityChecksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPortabilityChecksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPortabilityChecksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncPortabilityChecksResourceWithStreamingResponse(self)

    async def run(
        self,
        *,
        phone_numbers: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PortabilityCheckRunResponse:
        """
        Runs a portability check, returning the results immediately.

        Args:
          phone_numbers: The list of +E.164 formatted phone numbers to check for portability

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/portability_checks",
            body=await async_maybe_transform(
                {"phone_numbers": phone_numbers}, portability_check_run_params.PortabilityCheckRunParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PortabilityCheckRunResponse,
        )


class PortabilityChecksResourceWithRawResponse:
    def __init__(self, portability_checks: PortabilityChecksResource) -> None:
        self._portability_checks = portability_checks

        self.run = to_raw_response_wrapper(
            portability_checks.run,
        )


class AsyncPortabilityChecksResourceWithRawResponse:
    def __init__(self, portability_checks: AsyncPortabilityChecksResource) -> None:
        self._portability_checks = portability_checks

        self.run = async_to_raw_response_wrapper(
            portability_checks.run,
        )


class PortabilityChecksResourceWithStreamingResponse:
    def __init__(self, portability_checks: PortabilityChecksResource) -> None:
        self._portability_checks = portability_checks

        self.run = to_streamed_response_wrapper(
            portability_checks.run,
        )


class AsyncPortabilityChecksResourceWithStreamingResponse:
    def __init__(self, portability_checks: AsyncPortabilityChecksResource) -> None:
        self._portability_checks = portability_checks

        self.run = async_to_streamed_response_wrapper(
            portability_checks.run,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["InventoryCoverageListResponse", "Data", "Meta"]


class Data(BaseModel):
    administrative_area: Optional[str] = None

    advance_requirements: Optional[bool] = None
    """Indicates if the phone number requires advance requirements."""

    count: Optional[int] = None

    coverage_type: Optional[Literal["number", "block"]] = None

    group: Optional[str] = None

    group_type: Optional[str] = None

    number_range: Optional[int] = None

    number_type: Optional[Literal["did", "toll-free"]] = None

    phone_number_type: Optional[Literal["local", "toll_free", "national", "landline", "shared_cost", "mobile"]] = None

    record_type: Optional[str] = None


class Meta(BaseModel):
    total_results: Optional[int] = None


class InventoryCoverageListResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[Meta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel
from .mission_run_data import MissionRunData

__all__ = ["RunPauseRunResponse"]


class RunPauseRunResponse(BaseModel):
    data: MissionRunData



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    UsageReportListResponse,
    UsageReportGetOptionsResponse,
)
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestUsageReports:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        usage_report = client.usage_reports.list(
            dimensions=["string"],
            metrics=["string"],
            product="product",
        )
        assert_matches_type(SyncDefaultFlatPagination[UsageReportListResponse], usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        usage_report = client.usage_reports.list(
            dimensions=["string"],
            metrics=["string"],
            product="product",
            date_range="date_range",
            end_date="end_date",
            filter="filter",
            format="csv",
            managed_accounts=True,
            page_number=0,
            page_size=0,
            sort=["string"],
            start_date="start_date",
            authorization_bearer="authorization_bearer",
        )
        assert_matches_type(SyncDefaultFlatPagination[UsageReportListResponse], usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.usage_reports.with_raw_response.list(
            dimensions=["string"],
            metrics=["string"],
            product="product",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage_report = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[UsageReportListResponse], usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.usage_reports.with_streaming_response.list(
            dimensions=["string"],
            metrics=["string"],
            product="product",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage_report = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[UsageReportListResponse], usage_report, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_options(self, client: Telnyx) -> None:
        usage_report = client.usage_reports.get_options()
        assert_matches_type(UsageReportGetOptionsResponse, usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_get_options_with_all_params(self, client: Telnyx) -> None:
        usage_report = client.usage_reports.get_options(
            product="product",
            authorization_bearer="authorization_bearer",
        )
        assert_matches_type(UsageReportGetOptionsResponse, usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_get_options(self, client: Telnyx) -> None:
        response = client.usage_reports.with_raw_response.get_options()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage_report = response.parse()
        assert_matches_type(UsageReportGetOptionsResponse, usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_get_options(self, client: Telnyx) -> None:
        with client.usage_reports.with_streaming_response.get_options() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage_report = response.parse()
            assert_matches_type(UsageReportGetOptionsResponse, usage_report, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncUsageReports:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        usage_report = await async_client.usage_reports.list(
            dimensions=["string"],
            metrics=["string"],
            product="product",
        )
        assert_matches_type(AsyncDefaultFlatPagination[UsageReportListResponse], usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        usage_report = await async_client.usage_reports.list(
            dimensions=["string"],
            metrics=["string"],
            product="product",
            date_range="date_range",
            end_date="end_date",
            filter="filter",
            format="csv",
            managed_accounts=True,
            page_number=0,
            page_size=0,
            sort=["string"],
            start_date="start_date",
            authorization_bearer="authorization_bearer",
        )
        assert_matches_type(AsyncDefaultFlatPagination[UsageReportListResponse], usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.usage_reports.with_raw_response.list(
            dimensions=["string"],
            metrics=["string"],
            product="product",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage_report = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[UsageReportListResponse], usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.usage_reports.with_streaming_response.list(
            dimensions=["string"],
            metrics=["string"],
            product="product",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage_report = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[UsageReportListResponse], usage_report, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_options(self, async_client: AsyncTelnyx) -> None:
        usage_report = await async_client.usage_reports.get_options()
        assert_matches_type(UsageReportGetOptionsResponse, usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_get_options_with_all_params(self, async_client: AsyncTelnyx) -> None:
        usage_report = await async_client.usage_reports.get_options(
            product="product",
            authorization_bearer="authorization_bearer",
        )
        assert_matches_type(UsageReportGetOptionsResponse, usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_get_options(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.usage_reports.with_raw_response.get_options()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage_report = await response.parse()
        assert_matches_type(UsageReportGetOptionsResponse, usage_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_get_options(self, async_client: AsyncTelnyx) -> None:
        async with async_client.usage_reports.with_streaming_response.get_options() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage_report = await response.parse()
            assert_matches_type(UsageReportGetOptionsResponse, usage_report, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["VerifiedNumberCreateResponse"]


class VerifiedNumberCreateResponse(BaseModel):
    phone_number: Optional[str] = None

    verification_method: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import siprec_connector_create_params, siprec_connector_update_params
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.siprec_connector_create_response import SiprecConnectorCreateResponse
from ..types.siprec_connector_update_response import SiprecConnectorUpdateResponse
from ..types.siprec_connector_retrieve_response import SiprecConnectorRetrieveResponse

__all__ = ["SiprecConnectorsResource", "AsyncSiprecConnectorsResource"]


class SiprecConnectorsResource(SyncAPIResource):
    """SIPREC connectors configuration."""

    @cached_property
    def with_raw_response(self) -> SiprecConnectorsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return SiprecConnectorsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SiprecConnectorsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return SiprecConnectorsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        host: str,
        name: str,
        port: int,
        app_subdomain: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SiprecConnectorCreateResponse:
        """
        Creates a new SIPREC connector configuration.

        Args:
          host: Hostname/IPv4 address of the SIPREC SRS.

          name: Name for the SIPREC connector resource.

          port: Port for the SIPREC SRS.

          app_subdomain: Subdomain to route the call when using Telnyx SRS (optional for non-Telnyx SRS).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/siprec_connectors",
            body=maybe_transform(
                {
                    "host": host,
                    "name": name,
                    "port": port,
                    "app_subdomain": app_subdomain,
                },
                siprec_connector_create_params.SiprecConnectorCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SiprecConnectorCreateResponse,
        )

    def retrieve(
        self,
        connector_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SiprecConnectorRetrieveResponse:
        """
        Returns details of a stored SIPREC connector.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connector_name:
            raise ValueError(f"Expected a non-empty value for `connector_name` but received {connector_name!r}")
        return self._get(
            f"/siprec_connectors/{connector_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SiprecConnectorRetrieveResponse,
        )

    def update(
        self,
        connector_name: str,
        *,
        host: str,
        name: str,
        port: int,
        app_subdomain: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SiprecConnectorUpdateResponse:
        """
        Updates a stored SIPREC connector configuration.

        Args:
          host: Hostname/IPv4 address of the SIPREC SRS.

          name: Name for the SIPREC connector resource.

          port: Port for the SIPREC SRS.

          app_subdomain: Subdomain to route the call when using Telnyx SRS (optional for non-Telnyx SRS).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connector_name:
            raise ValueError(f"Expected a non-empty value for `connector_name` but received {connector_name!r}")
        return self._put(
            f"/siprec_connectors/{connector_name}",
            body=maybe_transform(
                {
                    "host": host,
                    "name": name,
                    "port": port,
                    "app_subdomain": app_subdomain,
                },
                siprec_connector_update_params.SiprecConnectorUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SiprecConnectorUpdateResponse,
        )

    def delete(
        self,
        connector_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes a stored SIPREC connector.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connector_name:
            raise ValueError(f"Expected a non-empty value for `connector_name` but received {connector_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/siprec_connectors/{connector_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncSiprecConnectorsResource(AsyncAPIResource):
    """SIPREC connectors configuration."""

    @cached_property
    def with_raw_response(self) -> AsyncSiprecConnectorsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSiprecConnectorsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSiprecConnectorsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncSiprecConnectorsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        host: str,
        name: str,
        port: int,
        app_subdomain: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SiprecConnectorCreateResponse:
        """
        Creates a new SIPREC connector configuration.

        Args:
          host: Hostname/IPv4 address of the SIPREC SRS.

          name: Name for the SIPREC connector resource.

          port: Port for the SIPREC SRS.

          app_subdomain: Subdomain to route the call when using Telnyx SRS (optional for non-Telnyx SRS).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/siprec_connectors",
            body=await async_maybe_transform(
                {
                    "host": host,
                    "name": name,
                    "port": port,
                    "app_subdomain": app_subdomain,
                },
                siprec_connector_create_params.SiprecConnectorCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SiprecConnectorCreateResponse,
        )

    async def retrieve(
        self,
        connector_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SiprecConnectorRetrieveResponse:
        """
        Returns details of a stored SIPREC connector.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connector_name:
            raise ValueError(f"Expected a non-empty value for `connector_name` but received {connector_name!r}")
        return await self._get(
            f"/siprec_connectors/{connector_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SiprecConnectorRetrieveResponse,
        )

    async def update(
        self,
        connector_name: str,
        *,
        host: str,
        name: str,
        port: int,
        app_subdomain: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SiprecConnectorUpdateResponse:
        """
        Updates a stored SIPREC connector configuration.

        Args:
          host: Hostname/IPv4 address of the SIPREC SRS.

          name: Name for the SIPREC connector resource.

          port: Port for the SIPREC SRS.

          app_subdomain: Subdomain to route the call when using Telnyx SRS (optional for non-Telnyx SRS).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connector_name:
            raise ValueError(f"Expected a non-empty value for `connector_name` but received {connector_name!r}")
        return await self._put(
            f"/siprec_connectors/{connector_name}",
            body=await async_maybe_transform(
                {
                    "host": host,
                    "name": name,
                    "port": port,
                    "app_subdomain": app_subdomain,
                },
                siprec_connector_update_params.SiprecConnectorUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SiprecConnectorUpdateResponse,
        )

    async def delete(
        self,
        connector_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes a stored SIPREC connector.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not connector_name:
            raise ValueError(f"Expected a non-empty value for `connector_name` but received {connector_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/siprec_connectors/{connector_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class SiprecConnectorsResourceWithRawResponse:
    def __init__(self, siprec_connectors: SiprecConnectorsResource) -> None:
        self._siprec_connectors = siprec_connectors

        self.create = to_raw_response_wrapper(
            siprec_connectors.create,
        )
        self.retrieve = to_raw_response_wrapper(
            siprec_connectors.retrieve,
        )
        self.update = to_raw_response_wrapper(
            siprec_connectors.update,
        )
        self.delete = to_raw_response_wrapper(
            siprec_connectors.delete,
        )


class AsyncSiprecConnectorsResourceWithRawResponse:
    def __init__(self, siprec_connectors: AsyncSiprecConnectorsResource) -> None:
        self._siprec_connectors = siprec_connectors

        self.create = async_to_raw_response_wrapper(
            siprec_connectors.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            siprec_connectors.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            siprec_connectors.update,
        )
        self.delete = async_to_raw_response_wrapper(
            siprec_connectors.delete,
        )


class SiprecConnectorsResourceWithStreamingResponse:
    def __init__(self, siprec_connectors: SiprecConnectorsResource) -> None:
        self._siprec_connectors = siprec_connectors

        self.create = to_streamed_response_wrapper(
            siprec_connectors.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            siprec_connectors.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            siprec_connectors.update,
        )
        self.delete = to_streamed_response_wrapper(
            siprec_connectors.delete,
        )


class AsyncSiprecConnectorsResourceWithStreamingResponse:
    def __init__(self, siprec_connectors: AsyncSiprecConnectorsResource) -> None:
        self._siprec_connectors = siprec_connectors

        self.create = async_to_streamed_response_wrapper(
            siprec_connectors.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            siprec_connectors.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            siprec_connectors.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            siprec_connectors.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["DynamicEmergencyAddress"]


class DynamicEmergencyAddress(BaseModel):
    administrative_area: str

    country_code: Literal["US", "CA", "PR"]

    house_number: str

    locality: str

    postal_code: str

    street_name: str

    id: Optional[str] = None

    created_at: Optional[str] = None
    """ISO 8601 formatted date of when the resource was created"""

    extended_address: Optional[str] = None

    house_suffix: Optional[str] = None

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    sip_geolocation_id: Optional[str] = None
    """
    Unique location reference string to be used in SIP INVITE from / p-asserted
    headers.
    """

    status: Optional[Literal["pending", "activated", "rejected"]] = None
    """Status of dynamic emergency address"""

    street_post_directional: Optional[str] = None

    street_pre_directional: Optional[str] = None

    street_suffix: Optional[str] = None

    updated_at: Optional[str] = None
    """ISO 8601 formatted date of when the resource was last updated"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .calls import (
    CallsResource,
    AsyncCallsResource,
    CallsResourceWithRawResponse,
    AsyncCallsResourceWithRawResponse,
    CallsResourceWithStreamingResponse,
    AsyncCallsResourceWithStreamingResponse,
)
from .siprec import (
    SiprecResource,
    AsyncSiprecResource,
    SiprecResourceWithRawResponse,
    AsyncSiprecResourceWithRawResponse,
    SiprecResourceWithStreamingResponse,
    AsyncSiprecResourceWithStreamingResponse,
)
from .streams import (
    StreamsResource,
    AsyncStreamsResource,
    StreamsResourceWithRawResponse,
    AsyncStreamsResourceWithRawResponse,
    StreamsResourceWithStreamingResponse,
    AsyncStreamsResourceWithStreamingResponse,
)
from .recordings import (
    RecordingsResource,
    AsyncRecordingsResource,
    RecordingsResourceWithRawResponse,
    AsyncRecordingsResourceWithRawResponse,
    RecordingsResourceWithStreamingResponse,
    AsyncRecordingsResourceWithStreamingResponse,
)
from .recordings_json import (
    RecordingsJsonResource,
    AsyncRecordingsJsonResource,
    RecordingsJsonResourceWithRawResponse,
    AsyncRecordingsJsonResourceWithRawResponse,
    RecordingsJsonResourceWithStreamingResponse,
    AsyncRecordingsJsonResourceWithStreamingResponse,
)

__all__ = [
    "RecordingsJsonResource",
    "AsyncRecordingsJsonResource",
    "RecordingsJsonResourceWithRawResponse",
    "AsyncRecordingsJsonResourceWithRawResponse",
    "RecordingsJsonResourceWithStreamingResponse",
    "AsyncRecordingsJsonResourceWithStreamingResponse",
    "RecordingsResource",
    "AsyncRecordingsResource",
    "RecordingsResourceWithRawResponse",
    "AsyncRecordingsResourceWithRawResponse",
    "RecordingsResourceWithStreamingResponse",
    "AsyncRecordingsResourceWithStreamingResponse",
    "SiprecResource",
    "AsyncSiprecResource",
    "SiprecResourceWithRawResponse",
    "AsyncSiprecResourceWithRawResponse",
    "SiprecResourceWithStreamingResponse",
    "AsyncSiprecResourceWithStreamingResponse",
    "StreamsResource",
    "AsyncStreamsResource",
    "StreamsResourceWithRawResponse",
    "AsyncStreamsResourceWithRawResponse",
    "StreamsResourceWithStreamingResponse",
    "AsyncStreamsResourceWithStreamingResponse",
    "CallsResource",
    "AsyncCallsResource",
    "CallsResourceWithRawResponse",
    "AsyncCallsResourceWithRawResponse",
    "CallsResourceWithStreamingResponse",
    "AsyncCallsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .shared.porting_orders_exception_type import PortingOrdersExceptionType

__all__ = ["PortingOrderRetrieveExceptionTypesResponse"]


class PortingOrderRetrieveExceptionTypesResponse(BaseModel):
    data: Optional[List[PortingOrdersExceptionType]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncPerPagePaginationV2, AsyncPerPagePaginationV2
from ..._base_client import AsyncPaginator, make_request_options
from ...types.messaging_10dlc import (
    phone_number_campaign_list_params,
    phone_number_campaign_create_params,
    phone_number_campaign_update_params,
)
from ...types.messaging_10dlc.phone_number_campaign import PhoneNumberCampaign

__all__ = ["PhoneNumberCampaignsResource", "AsyncPhoneNumberCampaignsResource"]


class PhoneNumberCampaignsResource(SyncAPIResource):
    """Phone number campaign assignment"""

    @cached_property
    def with_raw_response(self) -> PhoneNumberCampaignsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return PhoneNumberCampaignsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PhoneNumberCampaignsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return PhoneNumberCampaignsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        campaign_id: str,
        phone_number: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberCampaign:
        """
        Create New Phone Number Campaign

        Args:
          campaign_id: The ID of the campaign you want to link to the specified phone number.

          phone_number: The phone number you want to link to a specified campaign.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/10dlc/phone_number_campaigns",
            body=maybe_transform(
                {
                    "campaign_id": campaign_id,
                    "phone_number": phone_number,
                },
                phone_number_campaign_create_params.PhoneNumberCampaignCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberCampaign,
        )

    def retrieve(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberCampaign:
        """
        Retrieve an individual phone number/campaign assignment by `phoneNumber`.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return self._get(
            f"/10dlc/phone_number_campaigns/{phone_number}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberCampaign,
        )

    def update(
        self,
        campaign_phone_number: str,
        *,
        campaign_id: str,
        phone_number: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberCampaign:
        """
        Create New Phone Number Campaign

        Args:
          campaign_id: The ID of the campaign you want to link to the specified phone number.

          phone_number: The phone number you want to link to a specified campaign.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_phone_number:
            raise ValueError(
                f"Expected a non-empty value for `campaign_phone_number` but received {campaign_phone_number!r}"
            )
        return self._put(
            f"/10dlc/phone_number_campaigns/{campaign_phone_number}",
            body=maybe_transform(
                {
                    "campaign_id": campaign_id,
                    "phone_number": phone_number,
                },
                phone_number_campaign_update_params.PhoneNumberCampaignUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberCampaign,
        )

    def list(
        self,
        *,
        filter: phone_number_campaign_list_params.Filter | Omit = omit,
        page: int | Omit = omit,
        records_per_page: int | Omit = omit,
        sort: Literal["assignmentStatus", "-assignmentStatus", "createdAt", "-createdAt", "phoneNumber", "-phoneNumber"]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncPerPagePaginationV2[PhoneNumberCampaign]:
        """
        List phone number campaigns

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[telnyx_campaign_id], filter[telnyx_brand_id], filter[tcr_campaign_id],
              filter[tcr_brand_id]

          sort: Specifies the sort order for results. If not given, results are sorted by
              createdAt in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/10dlc/phone_number_campaigns",
            page=SyncPerPagePaginationV2[PhoneNumberCampaign],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                        "records_per_page": records_per_page,
                        "sort": sort,
                    },
                    phone_number_campaign_list_params.PhoneNumberCampaignListParams,
                ),
            ),
            model=PhoneNumberCampaign,
        )

    def delete(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberCampaign:
        """
        This endpoint allows you to remove a campaign assignment from the supplied
        `phoneNumber`.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return self._delete(
            f"/10dlc/phone_number_campaigns/{phone_number}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberCampaign,
        )


class AsyncPhoneNumberCampaignsResource(AsyncAPIResource):
    """Phone number campaign assignment"""

    @cached_property
    def with_raw_response(self) -> AsyncPhoneNumberCampaignsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPhoneNumberCampaignsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPhoneNumberCampaignsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncPhoneNumberCampaignsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        campaign_id: str,
        phone_number: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberCampaign:
        """
        Create New Phone Number Campaign

        Args:
          campaign_id: The ID of the campaign you want to link to the specified phone number.

          phone_number: The phone number you want to link to a specified campaign.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/10dlc/phone_number_campaigns",
            body=await async_maybe_transform(
                {
                    "campaign_id": campaign_id,
                    "phone_number": phone_number,
                },
                phone_number_campaign_create_params.PhoneNumberCampaignCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberCampaign,
        )

    async def retrieve(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberCampaign:
        """
        Retrieve an individual phone number/campaign assignment by `phoneNumber`.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return await self._get(
            f"/10dlc/phone_number_campaigns/{phone_number}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberCampaign,
        )

    async def update(
        self,
        campaign_phone_number: str,
        *,
        campaign_id: str,
        phone_number: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberCampaign:
        """
        Create New Phone Number Campaign

        Args:
          campaign_id: The ID of the campaign you want to link to the specified phone number.

          phone_number: The phone number you want to link to a specified campaign.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not campaign_phone_number:
            raise ValueError(
                f"Expected a non-empty value for `campaign_phone_number` but received {campaign_phone_number!r}"
            )
        return await self._put(
            f"/10dlc/phone_number_campaigns/{campaign_phone_number}",
            body=await async_maybe_transform(
                {
                    "campaign_id": campaign_id,
                    "phone_number": phone_number,
                },
                phone_number_campaign_update_params.PhoneNumberCampaignUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberCampaign,
        )

    def list(
        self,
        *,
        filter: phone_number_campaign_list_params.Filter | Omit = omit,
        page: int | Omit = omit,
        records_per_page: int | Omit = omit,
        sort: Literal["assignmentStatus", "-assignmentStatus", "createdAt", "-createdAt", "phoneNumber", "-phoneNumber"]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PhoneNumberCampaign, AsyncPerPagePaginationV2[PhoneNumberCampaign]]:
        """
        List phone number campaigns

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[telnyx_campaign_id], filter[telnyx_brand_id], filter[tcr_campaign_id],
              filter[tcr_brand_id]

          sort: Specifies the sort order for results. If not given, results are sorted by
              createdAt in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/10dlc/phone_number_campaigns",
            page=AsyncPerPagePaginationV2[PhoneNumberCampaign],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                        "records_per_page": records_per_page,
                        "sort": sort,
                    },
                    phone_number_campaign_list_params.PhoneNumberCampaignListParams,
                ),
            ),
            model=PhoneNumberCampaign,
        )

    async def delete(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PhoneNumberCampaign:
        """
        This endpoint allows you to remove a campaign assignment from the supplied
        `phoneNumber`.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return await self._delete(
            f"/10dlc/phone_number_campaigns/{phone_number}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PhoneNumberCampaign,
        )


class PhoneNumberCampaignsResourceWithRawResponse:
    def __init__(self, phone_number_campaigns: PhoneNumberCampaignsResource) -> None:
        self._phone_number_campaigns = phone_number_campaigns

        self.create = to_raw_response_wrapper(
            phone_number_campaigns.create,
        )
        self.retrieve = to_raw_response_wrapper(
            phone_number_campaigns.retrieve,
        )
        self.update = to_raw_response_wrapper(
            phone_number_campaigns.update,
        )
        self.list = to_raw_response_wrapper(
            phone_number_campaigns.list,
        )
        self.delete = to_raw_response_wrapper(
            phone_number_campaigns.delete,
        )


class AsyncPhoneNumberCampaignsResourceWithRawResponse:
    def __init__(self, phone_number_campaigns: AsyncPhoneNumberCampaignsResource) -> None:
        self._phone_number_campaigns = phone_number_campaigns

        self.create = async_to_raw_response_wrapper(
            phone_number_campaigns.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            phone_number_campaigns.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            phone_number_campaigns.update,
        )
        self.list = async_to_raw_response_wrapper(
            phone_number_campaigns.list,
        )
        self.delete = async_to_raw_response_wrapper(
            phone_number_campaigns.delete,
        )


class PhoneNumberCampaignsResourceWithStreamingResponse:
    def __init__(self, phone_number_campaigns: PhoneNumberCampaignsResource) -> None:
        self._phone_number_campaigns = phone_number_campaigns

        self.create = to_streamed_response_wrapper(
            phone_number_campaigns.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            phone_number_campaigns.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            phone_number_campaigns.update,
        )
        self.list = to_streamed_response_wrapper(
            phone_number_campaigns.list,
        )
        self.delete = to_streamed_response_wrapper(
            phone_number_campaigns.delete,
        )


class AsyncPhoneNumberCampaignsResourceWithStreamingResponse:
    def __init__(self, phone_number_campaigns: AsyncPhoneNumberCampaignsResource) -> None:
        self._phone_number_campaigns = phone_number_campaigns

        self.create = async_to_streamed_response_wrapper(
            phone_number_campaigns.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            phone_number_campaigns.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            phone_number_campaigns.update,
        )
        self.list = async_to_streamed_response_wrapper(
            phone_number_campaigns.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            phone_number_campaigns.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["PhoneNumberConfigurationListResponse"]


class PhoneNumberConfigurationListResponse(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies this phone number configuration"""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    porting_phone_number_id: Optional[str] = None
    """Identifies the associated porting phone number"""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""

    user_bundle_id: Optional[str] = None
    """Identifies the associated user bundle"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .settings import (
    SettingsResource,
    AsyncSettingsResource,
    SettingsResourceWithRawResponse,
    AsyncSettingsResourceWithRawResponse,
    SettingsResourceWithStreamingResponse,
    AsyncSettingsResourceWithStreamingResponse,
)
from .phone_numbers import (
    PhoneNumbersResource,
    AsyncPhoneNumbersResource,
    PhoneNumbersResourceWithRawResponse,
    AsyncPhoneNumbersResourceWithRawResponse,
    PhoneNumbersResourceWithStreamingResponse,
    AsyncPhoneNumbersResourceWithStreamingResponse,
)
from .business_accounts import (
    BusinessAccountsResource,
    AsyncBusinessAccountsResource,
    BusinessAccountsResourceWithRawResponse,
    AsyncBusinessAccountsResourceWithRawResponse,
    BusinessAccountsResourceWithStreamingResponse,
    AsyncBusinessAccountsResourceWithStreamingResponse,
)

__all__ = [
    "PhoneNumbersResource",
    "AsyncPhoneNumbersResource",
    "PhoneNumbersResourceWithRawResponse",
    "AsyncPhoneNumbersResourceWithRawResponse",
    "PhoneNumbersResourceWithStreamingResponse",
    "AsyncPhoneNumbersResourceWithStreamingResponse",
    "SettingsResource",
    "AsyncSettingsResource",
    "SettingsResourceWithRawResponse",
    "AsyncSettingsResourceWithRawResponse",
    "SettingsResourceWithStreamingResponse",
    "AsyncSettingsResourceWithStreamingResponse",
    "BusinessAccountsResource",
    "AsyncBusinessAccountsResource",
    "BusinessAccountsResourceWithRawResponse",
    "AsyncBusinessAccountsResourceWithRawResponse",
    "BusinessAccountsResourceWithStreamingResponse",
    "AsyncBusinessAccountsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ConversationListParams"]


class ConversationListParams(TypedDict, total=False):
    id: str
    """Filter by conversation ID (e.g. id=eq.123)"""

    created_at: str
    """Filter by creation datetime (e.g., `created_at=gte.2025-01-01`)"""

    last_message_at: str
    """Filter by last message datetime (e.g., `last_message_at=lte.2025-06-01`)"""

    limit: int
    """Limit the number of returned conversations (e.g., `limit=10`)"""

    metadata_assistant_id: Annotated[str, PropertyInfo(alias="metadata->assistant_id")]
    """Filter by assistant ID (e.g., `metadata->assistant_id=eq.assistant-123`)"""

    metadata_call_control_id: Annotated[str, PropertyInfo(alias="metadata->call_control_id")]
    """Filter by call control ID (e.g., `metadata->call_control_id=eq.v3:123`)"""

    metadata_telnyx_agent_target: Annotated[str, PropertyInfo(alias="metadata->telnyx_agent_target")]
    """
    Filter by the phone number, SIP URI, or other identifier for the agent (e.g.,
    `metadata->telnyx_agent_target=eq.+13128675309`)
    """

    metadata_telnyx_conversation_channel: Annotated[str, PropertyInfo(alias="metadata->telnyx_conversation_channel")]
    """
    Filter by conversation channel (e.g.,
    `metadata->telnyx_conversation_channel=eq.phone_call`)
    """

    metadata_telnyx_end_user_target: Annotated[str, PropertyInfo(alias="metadata->telnyx_end_user_target")]
    """
    Filter by the phone number, SIP URI, or other identifier for the end user (e.g.,
    `metadata->telnyx_end_user_target=eq.+13128675309`)
    """

    name: str
    """Filter by conversation Name (e.g. `name=like.Voice%`)"""

    or_: Annotated[str, PropertyInfo(alias="or")]
    """
    Apply OR conditions using PostgREST syntax (e.g.,
    `or=(created_at.gte.2025-04-01,last_message_at.gte.2025-04-01)`)
    """

    order: str
    """
    Order the results by specific fields (e.g., `order=created_at.desc` or
    `order=last_message_at.asc`)
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .metadata_field_mapping import MetadataFieldMapping as MetadataFieldMapping
from .child_relationship_info import ChildRelationshipInfo as ChildRelationshipInfo
from .parent_relationship_info import ParentRelationshipInfo as ParentRelationshipInfo
from .metadata_retrieve_response import MetadataRetrieveResponse as MetadataRetrieveResponse
from .metadata_retrieve_record_type_response import (
    MetadataRetrieveRecordTypeResponse as MetadataRetrieveRecordTypeResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ....._models import BaseModel
from .test_status import TestStatus

__all__ = ["TestRunDetailResult"]


class TestRunDetailResult(BaseModel):
    __test__ = False
    name: str

    status: TestStatus
    """Represents the lifecycle of a test:

    - 'pending': Test is waiting to be executed.
    - 'starting': Test execution is initializing.
    - 'running': Test is currently executing.
    - 'passed': Test completed successfully.
    - 'failed': Test executed but did not pass.
    - 'error': An error occurred during test execution.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .conference_command_result import ConferenceCommandResult

__all__ = ["ActionUpdateResponse"]


class ActionUpdateResponse(BaseModel):
    data: Optional[ConferenceCommandResult] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.phone_numbers import voicemail_create_params, voicemail_update_params
from ...types.phone_numbers.voicemail_create_response import VoicemailCreateResponse
from ...types.phone_numbers.voicemail_update_response import VoicemailUpdateResponse
from ...types.phone_numbers.voicemail_retrieve_response import VoicemailRetrieveResponse

__all__ = ["VoicemailResource", "AsyncVoicemailResource"]


class VoicemailResource(SyncAPIResource):
    """Voicemail API"""

    @cached_property
    def with_raw_response(self) -> VoicemailResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return VoicemailResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VoicemailResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return VoicemailResourceWithStreamingResponse(self)

    def create(
        self,
        phone_number_id: str,
        *,
        enabled: bool | Omit = omit,
        pin: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoicemailCreateResponse:
        """
        Create voicemail settings for a phone number

        Args:
          enabled: Whether voicemail is enabled.

          pin: The pin used for voicemail

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number_id:
            raise ValueError(f"Expected a non-empty value for `phone_number_id` but received {phone_number_id!r}")
        return self._post(
            f"/phone_numbers/{phone_number_id}/voicemail",
            body=maybe_transform(
                {
                    "enabled": enabled,
                    "pin": pin,
                },
                voicemail_create_params.VoicemailCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoicemailCreateResponse,
        )

    def retrieve(
        self,
        phone_number_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoicemailRetrieveResponse:
        """
        Returns the voicemail settings for a phone number

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number_id:
            raise ValueError(f"Expected a non-empty value for `phone_number_id` but received {phone_number_id!r}")
        return self._get(
            f"/phone_numbers/{phone_number_id}/voicemail",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoicemailRetrieveResponse,
        )

    def update(
        self,
        phone_number_id: str,
        *,
        enabled: bool | Omit = omit,
        pin: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoicemailUpdateResponse:
        """
        Update voicemail settings for a phone number

        Args:
          enabled: Whether voicemail is enabled.

          pin: The pin used for voicemail

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number_id:
            raise ValueError(f"Expected a non-empty value for `phone_number_id` but received {phone_number_id!r}")
        return self._patch(
            f"/phone_numbers/{phone_number_id}/voicemail",
            body=maybe_transform(
                {
                    "enabled": enabled,
                    "pin": pin,
                },
                voicemail_update_params.VoicemailUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoicemailUpdateResponse,
        )


class AsyncVoicemailResource(AsyncAPIResource):
    """Voicemail API"""

    @cached_property
    def with_raw_response(self) -> AsyncVoicemailResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncVoicemailResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVoicemailResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncVoicemailResourceWithStreamingResponse(self)

    async def create(
        self,
        phone_number_id: str,
        *,
        enabled: bool | Omit = omit,
        pin: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoicemailCreateResponse:
        """
        Create voicemail settings for a phone number

        Args:
          enabled: Whether voicemail is enabled.

          pin: The pin used for voicemail

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number_id:
            raise ValueError(f"Expected a non-empty value for `phone_number_id` but received {phone_number_id!r}")
        return await self._post(
            f"/phone_numbers/{phone_number_id}/voicemail",
            body=await async_maybe_transform(
                {
                    "enabled": enabled,
                    "pin": pin,
                },
                voicemail_create_params.VoicemailCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoicemailCreateResponse,
        )

    async def retrieve(
        self,
        phone_number_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoicemailRetrieveResponse:
        """
        Returns the voicemail settings for a phone number

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number_id:
            raise ValueError(f"Expected a non-empty value for `phone_number_id` but received {phone_number_id!r}")
        return await self._get(
            f"/phone_numbers/{phone_number_id}/voicemail",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoicemailRetrieveResponse,
        )

    async def update(
        self,
        phone_number_id: str,
        *,
        enabled: bool | Omit = omit,
        pin: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VoicemailUpdateResponse:
        """
        Update voicemail settings for a phone number

        Args:
          enabled: Whether voicemail is enabled.

          pin: The pin used for voicemail

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number_id:
            raise ValueError(f"Expected a non-empty value for `phone_number_id` but received {phone_number_id!r}")
        return await self._patch(
            f"/phone_numbers/{phone_number_id}/voicemail",
            body=await async_maybe_transform(
                {
                    "enabled": enabled,
                    "pin": pin,
                },
                voicemail_update_params.VoicemailUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VoicemailUpdateResponse,
        )


class VoicemailResourceWithRawResponse:
    def __init__(self, voicemail: VoicemailResource) -> None:
        self._voicemail = voicemail

        self.create = to_raw_response_wrapper(
            voicemail.create,
        )
        self.retrieve = to_raw_response_wrapper(
            voicemail.retrieve,
        )
        self.update = to_raw_response_wrapper(
            voicemail.update,
        )


class AsyncVoicemailResourceWithRawResponse:
    def __init__(self, voicemail: AsyncVoicemailResource) -> None:
        self._voicemail = voicemail

        self.create = async_to_raw_response_wrapper(
            voicemail.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            voicemail.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            voicemail.update,
        )


class VoicemailResourceWithStreamingResponse:
    def __init__(self, voicemail: VoicemailResource) -> None:
        self._voicemail = voicemail

        self.create = to_streamed_response_wrapper(
            voicemail.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            voicemail.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            voicemail.update,
        )


class AsyncVoicemailResourceWithStreamingResponse:
    def __init__(self, voicemail: AsyncVoicemailResource) -> None:
        self._voicemail = voicemail

        self.create = async_to_streamed_response_wrapper(
            voicemail.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            voicemail.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            voicemail.update,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, TypedDict

__all__ = ["TemplateCreateParams"]


class TemplateCreateParams(TypedDict, total=False):
    category: Required[Literal["MARKETING", "UTILITY", "AUTHENTICATION"]]

    components: Required[Iterable[object]]

    language: Required[str]

    name: Required[str]

    waba_id: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from .user_bundle_resource import UserBundleResource

__all__ = ["UserBundleListResourcesResponse"]


class UserBundleListResourcesResponse(BaseModel):
    data: List[UserBundleResource]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo

__all__ = [
    "MobilePushCredentialCreateParams",
    "CreateMobilePushCredentialRequest",
    "CreateMobilePushCredentialRequestIos",
    "CreateMobilePushCredentialRequestAndroid",
]


class MobilePushCredentialCreateParams(TypedDict, total=False):
    create_mobile_push_credential_request: Required[
        Annotated[CreateMobilePushCredentialRequest, PropertyInfo(alias="createMobilePushCredentialRequest")]
    ]


class CreateMobilePushCredentialRequestIos(TypedDict, total=False):
    alias: Required[str]
    """Alias to uniquely identify the credential"""

    certificate: Required[str]
    """Certificate as received from APNs"""

    private_key: Required[str]
    """Corresponding private key to the certificate as received from APNs"""

    type: Required[Literal["ios"]]
    """Type of mobile push credential. Should be <code>ios</code> here"""


class CreateMobilePushCredentialRequestAndroid(TypedDict, total=False):
    alias: Required[str]
    """Alias to uniquely identify the credential"""

    project_account_json_file: Required[Dict[str, object]]
    """Private key file in JSON format"""

    type: Required[Literal["android"]]
    """Type of mobile push credential. Should be <code>android</code> here"""


CreateMobilePushCredentialRequest: TypeAlias = Union[
    CreateMobilePushCredentialRequestIos, CreateMobilePushCredentialRequestAndroid
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["Volume"]

Volume: TypeAlias = Literal[
    "10", "100", "1,000", "10,000", "100,000", "250,000", "500,000", "750,000", "1,000,000", "5,000,000", "10,000,000+"
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .dynamic_emergency_address import DynamicEmergencyAddress

__all__ = ["DynamicEmergencyAddressCreateResponse"]


class DynamicEmergencyAddressCreateResponse(BaseModel):
    data: Optional[DynamicEmergencyAddress] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["DetailRecordListParams", "Filter"]


class DetailRecordListParams(TypedDict, total=False):
    filter: Filter
    """Filter records on a given record attribute and value.

    <br/>Example: filter[status]=delivered. <br/>Required: filter[record_type] must
    be specified.
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]

    sort: SequenceNotStr[str]
    """Specifies the sort order for results. <br/>Example: sort=-created_at"""


class Filter(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    """Filter records on a given record attribute and value.

    <br/>Example: filter[status]=delivered. <br/>Required: filter[record_type] must be specified.
    """

    record_type: Required[
        Literal[
            "ai-voice-assistant",
            "amd",
            "call-control",
            "conference",
            "conference-participant",
            "embedding",
            "fax",
            "inference",
            "inference-speech-to-text",
            "media_storage",
            "media-streaming",
            "messaging",
            "noise-suppression",
            "recording",
            "sip-trunking",
            "siprec-client",
            "stt",
            "tts",
            "verify",
            "webrtc",
            "wireless",
        ]
    ]
    """Filter by the given record type."""

    date_range: Literal[
        "yesterday",
        "today",
        "tomorrow",
        "last_week",
        "this_week",
        "next_week",
        "last_month",
        "this_month",
        "next_month",
    ]
    """Filter by the given user-friendly date range.

    You can specify one of the following enum values, or a dynamic one using this
    format: last_N_days.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .credential_connection import CredentialConnection

__all__ = ["CredentialConnectionCreateResponse"]


class CredentialConnectionCreateResponse(BaseModel):
    data: Optional[CredentialConnection] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ...._base_client import AsyncPaginator, make_request_options
from .profile.profile import (
    ProfileResource,
    AsyncProfileResource,
    ProfileResourceWithRawResponse,
    AsyncProfileResourceWithRawResponse,
    ProfileResourceWithStreamingResponse,
    AsyncProfileResourceWithStreamingResponse,
)
from .calling_settings import (
    CallingSettingsResource,
    AsyncCallingSettingsResource,
    CallingSettingsResourceWithRawResponse,
    AsyncCallingSettingsResourceWithRawResponse,
    CallingSettingsResourceWithStreamingResponse,
    AsyncCallingSettingsResourceWithStreamingResponse,
)
from ....types.whatsapp import (
    phone_number_list_params,
    phone_number_verify_params,
    phone_number_resend_verification_params,
)
from ....types.whatsapp.phone_number_list_response import PhoneNumberListResponse

__all__ = ["PhoneNumbersResource", "AsyncPhoneNumbersResource"]


class PhoneNumbersResource(SyncAPIResource):
    """Manage Whatsapp phone numbers"""

    @cached_property
    def calling_settings(self) -> CallingSettingsResource:
        """Manage Whatsapp phone numbers"""
        return CallingSettingsResource(self._client)

    @cached_property
    def profile(self) -> ProfileResource:
        """Manage Whatsapp phone numbers"""
        return ProfileResource(self._client)

    @cached_property
    def with_raw_response(self) -> PhoneNumbersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return PhoneNumbersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PhoneNumbersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return PhoneNumbersResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[PhoneNumberListResponse]:
        """
        List Whatsapp phone numbers

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v2/whatsapp/phone_numbers",
            page=SyncDefaultFlatPagination[PhoneNumberListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    phone_number_list_params.PhoneNumberListParams,
                ),
            ),
            model=PhoneNumberListResponse,
        )

    def delete(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a Whatsapp phone number

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/v2/whatsapp/phone_numbers/{phone_number}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def resend_verification(
        self,
        phone_number: str,
        *,
        verification_method: Literal["sms", "voice"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Resend verification code

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/v2/whatsapp/phone_numbers/{phone_number}/resend_verification",
            body=maybe_transform(
                {"verification_method": verification_method},
                phone_number_resend_verification_params.PhoneNumberResendVerificationParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def verify(
        self,
        phone_number: str,
        *,
        code: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Submit verification code for a phone number

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/v2/whatsapp/phone_numbers/{phone_number}/verify",
            body=maybe_transform({"code": code}, phone_number_verify_params.PhoneNumberVerifyParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncPhoneNumbersResource(AsyncAPIResource):
    """Manage Whatsapp phone numbers"""

    @cached_property
    def calling_settings(self) -> AsyncCallingSettingsResource:
        """Manage Whatsapp phone numbers"""
        return AsyncCallingSettingsResource(self._client)

    @cached_property
    def profile(self) -> AsyncProfileResource:
        """Manage Whatsapp phone numbers"""
        return AsyncProfileResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncPhoneNumbersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPhoneNumbersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPhoneNumbersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncPhoneNumbersResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PhoneNumberListResponse, AsyncDefaultFlatPagination[PhoneNumberListResponse]]:
        """
        List Whatsapp phone numbers

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v2/whatsapp/phone_numbers",
            page=AsyncDefaultFlatPagination[PhoneNumberListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    phone_number_list_params.PhoneNumberListParams,
                ),
            ),
            model=PhoneNumberListResponse,
        )

    async def delete(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a Whatsapp phone number

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/v2/whatsapp/phone_numbers/{phone_number}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def resend_verification(
        self,
        phone_number: str,
        *,
        verification_method: Literal["sms", "voice"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Resend verification code

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/v2/whatsapp/phone_numbers/{phone_number}/resend_verification",
            body=await async_maybe_transform(
                {"verification_method": verification_method},
                phone_number_resend_verification_params.PhoneNumberResendVerificationParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def verify(
        self,
        phone_number: str,
        *,
        code: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Submit verification code for a phone number

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/v2/whatsapp/phone_numbers/{phone_number}/verify",
            body=await async_maybe_transform({"code": code}, phone_number_verify_params.PhoneNumberVerifyParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class PhoneNumbersResourceWithRawResponse:
    def __init__(self, phone_numbers: PhoneNumbersResource) -> None:
        self._phone_numbers = phone_numbers

        self.list = to_raw_response_wrapper(
            phone_numbers.list,
        )
        self.delete = to_raw_response_wrapper(
            phone_numbers.delete,
        )
        self.resend_verification = to_raw_response_wrapper(
            phone_numbers.resend_verification,
        )
        self.verify = to_raw_response_wrapper(
            phone_numbers.verify,
        )

    @cached_property
    def calling_settings(self) -> CallingSettingsResourceWithRawResponse:
        """Manage Whatsapp phone numbers"""
        return CallingSettingsResourceWithRawResponse(self._phone_numbers.calling_settings)

    @cached_property
    def profile(self) -> ProfileResourceWithRawResponse:
        """Manage Whatsapp phone numbers"""
        return ProfileResourceWithRawResponse(self._phone_numbers.profile)


class AsyncPhoneNumbersResourceWithRawResponse:
    def __init__(self, phone_numbers: AsyncPhoneNumbersResource) -> None:
        self._phone_numbers = phone_numbers

        self.list = async_to_raw_response_wrapper(
            phone_numbers.list,
        )
        self.delete = async_to_raw_response_wrapper(
            phone_numbers.delete,
        )
        self.resend_verification = async_to_raw_response_wrapper(
            phone_numbers.resend_verification,
        )
        self.verify = async_to_raw_response_wrapper(
            phone_numbers.verify,
        )

    @cached_property
    def calling_settings(self) -> AsyncCallingSettingsResourceWithRawResponse:
        """Manage Whatsapp phone numbers"""
        return AsyncCallingSettingsResourceWithRawResponse(self._phone_numbers.calling_settings)

    @cached_property
    def profile(self) -> AsyncProfileResourceWithRawResponse:
        """Manage Whatsapp phone numbers"""
        return AsyncProfileResourceWithRawResponse(self._phone_numbers.profile)


class PhoneNumbersResourceWithStreamingResponse:
    def __init__(self, phone_numbers: PhoneNumbersResource) -> None:
        self._phone_numbers = phone_numbers

        self.list = to_streamed_response_wrapper(
            phone_numbers.list,
        )
        self.delete = to_streamed_response_wrapper(
            phone_numbers.delete,
        )
        self.resend_verification = to_streamed_response_wrapper(
            phone_numbers.resend_verification,
        )
        self.verify = to_streamed_response_wrapper(
            phone_numbers.verify,
        )

    @cached_property
    def calling_settings(self) -> CallingSettingsResourceWithStreamingResponse:
        """Manage Whatsapp phone numbers"""
        return CallingSettingsResourceWithStreamingResponse(self._phone_numbers.calling_settings)

    @cached_property
    def profile(self) -> ProfileResourceWithStreamingResponse:
        """Manage Whatsapp phone numbers"""
        return ProfileResourceWithStreamingResponse(self._phone_numbers.profile)


class AsyncPhoneNumbersResourceWithStreamingResponse:
    def __init__(self, phone_numbers: AsyncPhoneNumbersResource) -> None:
        self._phone_numbers = phone_numbers

        self.list = async_to_streamed_response_wrapper(
            phone_numbers.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            phone_numbers.delete,
        )
        self.resend_verification = async_to_streamed_response_wrapper(
            phone_numbers.resend_verification,
        )
        self.verify = async_to_streamed_response_wrapper(
            phone_numbers.verify,
        )

    @cached_property
    def calling_settings(self) -> AsyncCallingSettingsResourceWithStreamingResponse:
        """Manage Whatsapp phone numbers"""
        return AsyncCallingSettingsResourceWithStreamingResponse(self._phone_numbers.calling_settings)

    @cached_property
    def profile(self) -> AsyncProfileResourceWithStreamingResponse:
        """Manage Whatsapp phone numbers"""
        return AsyncProfileResourceWithStreamingResponse(self._phone_numbers.profile)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Query, Headers, NoneType, NotGiven, not_given
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.ai.embeddings.bucket_list_response import BucketListResponse
from ....types.ai.embeddings.bucket_retrieve_response import BucketRetrieveResponse

__all__ = ["BucketsResource", "AsyncBucketsResource"]


class BucketsResource(SyncAPIResource):
    """Embed documents and perform text searches"""

    @cached_property
    def with_raw_response(self) -> BucketsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return BucketsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BucketsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return BucketsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        bucket_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BucketRetrieveResponse:
        """
        Get all embedded files for a given user bucket, including their processing
        status.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        return self._get(
            f"/ai/embeddings/buckets/{bucket_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BucketRetrieveResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BucketListResponse:
        """Get all embedding buckets for a user."""
        return self._get(
            "/ai/embeddings/buckets",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BucketListResponse,
        )

    def delete(
        self,
        bucket_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes an entire bucket's embeddings and disables the bucket for AI-use,
        returning it to normal storage pricing.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/ai/embeddings/buckets/{bucket_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncBucketsResource(AsyncAPIResource):
    """Embed documents and perform text searches"""

    @cached_property
    def with_raw_response(self) -> AsyncBucketsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBucketsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBucketsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncBucketsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        bucket_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BucketRetrieveResponse:
        """
        Get all embedded files for a given user bucket, including their processing
        status.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        return await self._get(
            f"/ai/embeddings/buckets/{bucket_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BucketRetrieveResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BucketListResponse:
        """Get all embedding buckets for a user."""
        return await self._get(
            "/ai/embeddings/buckets",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BucketListResponse,
        )

    async def delete(
        self,
        bucket_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Deletes an entire bucket's embeddings and disables the bucket for AI-use,
        returning it to normal storage pricing.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not bucket_name:
            raise ValueError(f"Expected a non-empty value for `bucket_name` but received {bucket_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/ai/embeddings/buckets/{bucket_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class BucketsResourceWithRawResponse:
    def __init__(self, buckets: BucketsResource) -> None:
        self._buckets = buckets

        self.retrieve = to_raw_response_wrapper(
            buckets.retrieve,
        )
        self.list = to_raw_response_wrapper(
            buckets.list,
        )
        self.delete = to_raw_response_wrapper(
            buckets.delete,
        )


class AsyncBucketsResourceWithRawResponse:
    def __init__(self, buckets: AsyncBucketsResource) -> None:
        self._buckets = buckets

        self.retrieve = async_to_raw_response_wrapper(
            buckets.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            buckets.list,
        )
        self.delete = async_to_raw_response_wrapper(
            buckets.delete,
        )


class BucketsResourceWithStreamingResponse:
    def __init__(self, buckets: BucketsResource) -> None:
        self._buckets = buckets

        self.retrieve = to_streamed_response_wrapper(
            buckets.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            buckets.list,
        )
        self.delete = to_streamed_response_wrapper(
            buckets.delete,
        )


class AsyncBucketsResourceWithStreamingResponse:
    def __init__(self, buckets: AsyncBucketsResource) -> None:
        self._buckets = buckets

        self.retrieve = async_to_streamed_response_wrapper(
            buckets.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            buckets.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            buckets.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime

import httpx

from ....._types import Body, Query, Headers, NotGiven, not_given
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.legacy.reporting.batch_detail_records import speech_to_text_create_params
from .....types.legacy.reporting.batch_detail_records.speech_to_text_list_response import SpeechToTextListResponse
from .....types.legacy.reporting.batch_detail_records.speech_to_text_create_response import SpeechToTextCreateResponse
from .....types.legacy.reporting.batch_detail_records.speech_to_text_delete_response import SpeechToTextDeleteResponse
from .....types.legacy.reporting.batch_detail_records.speech_to_text_retrieve_response import (
    SpeechToTextRetrieveResponse,
)

__all__ = ["SpeechToTextResource", "AsyncSpeechToTextResource"]


class SpeechToTextResource(SyncAPIResource):
    """Speech to text batch detail records"""

    @cached_property
    def with_raw_response(self) -> SpeechToTextResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return SpeechToTextResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SpeechToTextResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return SpeechToTextResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        end_date: Union[str, datetime],
        start_date: Union[str, datetime],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpeechToTextCreateResponse:
        """
        Creates a new Speech to Text batch report request with the specified filters

        Args:
          end_date: End date in ISO format with timezone (date range must be up to one month)

          start_date: Start date in ISO format with timezone

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/legacy/reporting/batch_detail_records/speech_to_text",
            body=maybe_transform(
                {
                    "end_date": end_date,
                    "start_date": start_date,
                },
                speech_to_text_create_params.SpeechToTextCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpeechToTextCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpeechToTextRetrieveResponse:
        """
        Retrieves a specific Speech to Text batch report request by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/legacy/reporting/batch_detail_records/speech_to_text/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpeechToTextRetrieveResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpeechToTextListResponse:
        """Retrieves all Speech to Text batch report requests for the authenticated user"""
        return self._get(
            "/legacy/reporting/batch_detail_records/speech_to_text",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpeechToTextListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpeechToTextDeleteResponse:
        """
        Deletes a specific Speech to Text batch report request by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/legacy/reporting/batch_detail_records/speech_to_text/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpeechToTextDeleteResponse,
        )


class AsyncSpeechToTextResource(AsyncAPIResource):
    """Speech to text batch detail records"""

    @cached_property
    def with_raw_response(self) -> AsyncSpeechToTextResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSpeechToTextResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSpeechToTextResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncSpeechToTextResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        end_date: Union[str, datetime],
        start_date: Union[str, datetime],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpeechToTextCreateResponse:
        """
        Creates a new Speech to Text batch report request with the specified filters

        Args:
          end_date: End date in ISO format with timezone (date range must be up to one month)

          start_date: Start date in ISO format with timezone

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/legacy/reporting/batch_detail_records/speech_to_text",
            body=await async_maybe_transform(
                {
                    "end_date": end_date,
                    "start_date": start_date,
                },
                speech_to_text_create_params.SpeechToTextCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpeechToTextCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpeechToTextRetrieveResponse:
        """
        Retrieves a specific Speech to Text batch report request by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/legacy/reporting/batch_detail_records/speech_to_text/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpeechToTextRetrieveResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpeechToTextListResponse:
        """Retrieves all Speech to Text batch report requests for the authenticated user"""
        return await self._get(
            "/legacy/reporting/batch_detail_records/speech_to_text",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpeechToTextListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SpeechToTextDeleteResponse:
        """
        Deletes a specific Speech to Text batch report request by ID

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/legacy/reporting/batch_detail_records/speech_to_text/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SpeechToTextDeleteResponse,
        )


class SpeechToTextResourceWithRawResponse:
    def __init__(self, speech_to_text: SpeechToTextResource) -> None:
        self._speech_to_text = speech_to_text

        self.create = to_raw_response_wrapper(
            speech_to_text.create,
        )
        self.retrieve = to_raw_response_wrapper(
            speech_to_text.retrieve,
        )
        self.list = to_raw_response_wrapper(
            speech_to_text.list,
        )
        self.delete = to_raw_response_wrapper(
            speech_to_text.delete,
        )


class AsyncSpeechToTextResourceWithRawResponse:
    def __init__(self, speech_to_text: AsyncSpeechToTextResource) -> None:
        self._speech_to_text = speech_to_text

        self.create = async_to_raw_response_wrapper(
            speech_to_text.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            speech_to_text.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            speech_to_text.list,
        )
        self.delete = async_to_raw_response_wrapper(
            speech_to_text.delete,
        )


class SpeechToTextResourceWithStreamingResponse:
    def __init__(self, speech_to_text: SpeechToTextResource) -> None:
        self._speech_to_text = speech_to_text

        self.create = to_streamed_response_wrapper(
            speech_to_text.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            speech_to_text.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            speech_to_text.list,
        )
        self.delete = to_streamed_response_wrapper(
            speech_to_text.delete,
        )


class AsyncSpeechToTextResourceWithStreamingResponse:
    def __init__(self, speech_to_text: AsyncSpeechToTextResource) -> None:
        self._speech_to_text = speech_to_text

        self.create = async_to_streamed_response_wrapper(
            speech_to_text.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            speech_to_text.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            speech_to_text.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            speech_to_text.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .._types import Body, Query, Headers, NotGiven, not_given
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.global_ip_protocol_list_response import GlobalIPProtocolListResponse

__all__ = ["GlobalIPProtocolsResource", "AsyncGlobalIPProtocolsResource"]


class GlobalIPProtocolsResource(SyncAPIResource):
    """Global IPs"""

    @cached_property
    def with_raw_response(self) -> GlobalIPProtocolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return GlobalIPProtocolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GlobalIPProtocolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return GlobalIPProtocolsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPProtocolListResponse:
        """List all Global IP Protocols"""
        return self._get(
            "/global_ip_protocols",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GlobalIPProtocolListResponse,
        )


class AsyncGlobalIPProtocolsResource(AsyncAPIResource):
    """Global IPs"""

    @cached_property
    def with_raw_response(self) -> AsyncGlobalIPProtocolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncGlobalIPProtocolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGlobalIPProtocolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncGlobalIPProtocolsResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPProtocolListResponse:
        """List all Global IP Protocols"""
        return await self._get(
            "/global_ip_protocols",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GlobalIPProtocolListResponse,
        )


class GlobalIPProtocolsResourceWithRawResponse:
    def __init__(self, global_ip_protocols: GlobalIPProtocolsResource) -> None:
        self._global_ip_protocols = global_ip_protocols

        self.list = to_raw_response_wrapper(
            global_ip_protocols.list,
        )


class AsyncGlobalIPProtocolsResourceWithRawResponse:
    def __init__(self, global_ip_protocols: AsyncGlobalIPProtocolsResource) -> None:
        self._global_ip_protocols = global_ip_protocols

        self.list = async_to_raw_response_wrapper(
            global_ip_protocols.list,
        )


class GlobalIPProtocolsResourceWithStreamingResponse:
    def __init__(self, global_ip_protocols: GlobalIPProtocolsResource) -> None:
        self._global_ip_protocols = global_ip_protocols

        self.list = to_streamed_response_wrapper(
            global_ip_protocols.list,
        )


class AsyncGlobalIPProtocolsResourceWithStreamingResponse:
    def __init__(self, global_ip_protocols: AsyncGlobalIPProtocolsResource) -> None:
        self._global_ip_protocols = global_ip_protocols

        self.list = async_to_streamed_response_wrapper(
            global_ip_protocols.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["DocServiceDocument", "Size"]


class Size(BaseModel):
    """Indicates the document's filesize"""

    amount: Optional[int] = None
    """The number of bytes"""

    unit: Optional[str] = None
    """Identifies the unit"""


class DocServiceDocument(BaseModel):
    id: Optional[str] = None
    """Identifies the resource."""

    av_scan_status: Optional[Literal["scanned", "infected", "pending_scan", "not_scanned"]] = None
    """The antivirus scan status of the document."""

    content_type: Optional[str] = None
    """The document's content_type."""

    created_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was created."""

    customer_reference: Optional[str] = None
    """Optional reference string for customer tracking."""

    filename: Optional[str] = None
    """The filename of the document."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    sha256: Optional[str] = None
    """The document's SHA256 hash provided for optional verification purposes."""

    size: Optional[Size] = None
    """Indicates the document's filesize"""

    status: Optional[Literal["pending", "verified", "denied"]] = None
    """Indicates the current document reviewing status"""

    updated_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["MessagingURLDomainListResponse"]


class MessagingURLDomainListResponse(BaseModel):
    id: Optional[str] = None

    record_type: Optional[str] = None

    url_domain: Optional[str] = None

    use_case: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .phone_number_detailed import PhoneNumberDetailed

__all__ = ["PhoneNumberUpdateResponse"]


class PhoneNumberUpdateResponse(BaseModel):
    data: Optional[PhoneNumberDetailed] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["CustomerServiceRecord", "Result", "ResultAddress", "ResultAdmin"]


class ResultAddress(BaseModel):
    """The address of the customer service record"""

    administrative_area: Optional[str] = None
    """The state of the address"""

    full_address: Optional[str] = None
    """The full address"""

    locality: Optional[str] = None
    """The city of the address"""

    postal_code: Optional[str] = None
    """The zip code of the address"""

    street_address: Optional[str] = None
    """The street address"""


class ResultAdmin(BaseModel):
    """The admin of the customer service record."""

    account_number: Optional[str] = None
    """The account number of the customer service record."""

    authorized_person_name: Optional[str] = None
    """The authorized person name of the customer service record."""

    billing_phone_number: Optional[str] = None
    """The billing phone number of the customer service record."""

    name: Optional[str] = None
    """The name of the customer service record."""


class Result(BaseModel):
    """The result of the CSR request.

    This field would be null in case of `pending` or `failed` status.
    """

    address: Optional[ResultAddress] = None
    """The address of the customer service record"""

    admin: Optional[ResultAdmin] = None
    """The admin of the customer service record."""

    associated_phone_numbers: Optional[List[str]] = None
    """The associated phone numbers of the customer service record."""

    carrier_name: Optional[str] = None
    """The name of the carrier that the customer service record is for."""


class CustomerServiceRecord(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies this customer service record"""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    error_message: Optional[str] = None
    """The error message in case status is `failed`.

    This field would be null in case of `pending` or `completed` status.
    """

    phone_number: Optional[str] = None
    """The phone number of the customer service record."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    result: Optional[Result] = None
    """The result of the CSR request.

    This field would be null in case of `pending` or `failed` status.
    """

    status: Optional[Literal["pending", "completed", "failed"]] = None
    """The status of the customer service record"""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    webhook_url: Optional[str] = None
    """Callback URL to receive webhook notifications."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["CostInformation"]


class CostInformation(BaseModel):
    currency: Optional[str] = None
    """The ISO 4217 code for the currency."""

    monthly_cost: Optional[str] = None

    upfront_cost: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["CdrUsageReportFetchSyncResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """Identifies the resource"""

    aggregation_type: Optional[Literal["NO_AGGREGATION", "CONNECTION", "TAG", "BILLING_GROUP"]] = None

    connections: Optional[List[int]] = None

    created_at: Optional[datetime] = None

    end_time: Optional[datetime] = None

    product_breakdown: Optional[
        Literal["NO_BREAKDOWN", "DID_VS_TOLL_FREE", "COUNTRY", "DID_VS_TOLL_FREE_PER_COUNTRY"]
    ] = None

    record_type: Optional[str] = None

    report_url: Optional[str] = None

    result: Optional[Dict[str, object]] = None

    start_time: Optional[datetime] = None

    status: Optional[Literal["PENDING", "COMPLETE", "FAILED", "EXPIRED"]] = None

    updated_at: Optional[datetime] = None


class CdrUsageReportFetchSyncResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .transcription_settings_config import TranscriptionSettingsConfig

__all__ = ["TranscriptionSettings"]


class TranscriptionSettings(BaseModel):
    language: Optional[str] = None
    """The language of the audio to be transcribed.

    If not set, of if set to `auto`, the model will automatically detect the
    language.
    """

    model: Optional[
        Literal[
            "deepgram/flux",
            "deepgram/nova-3",
            "deepgram/nova-2",
            "azure/fast",
            "distil-whisper/distil-large-v2",
            "openai/whisper-large-v3-turbo",
        ]
    ] = None
    """The speech to text model to be used by the voice assistant.

    All the deepgram models are run on-premise.

    - `deepgram/flux` is optimized for turn-taking but is English-only.
    - `deepgram/nova-3` is multi-lingual with automatic language detection but
      slightly higher latency.
    """

    region: Optional[str] = None
    """
    Region on third party cloud providers (currently Azure) if using one of their
    models
    """

    settings: Optional[TranscriptionSettingsConfig] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .settings import (
    SettingsResource,
    AsyncSettingsResource,
    SettingsResourceWithRawResponse,
    AsyncSettingsResourceWithRawResponse,
    SettingsResourceWithStreamingResponse,
    AsyncSettingsResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ...._utils import maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .phone_numbers import (
    PhoneNumbersResource,
    AsyncPhoneNumbersResource,
    PhoneNumbersResourceWithRawResponse,
    AsyncPhoneNumbersResourceWithRawResponse,
    PhoneNumbersResourceWithStreamingResponse,
    AsyncPhoneNumbersResourceWithStreamingResponse,
)
from ...._base_client import AsyncPaginator, make_request_options
from ....types.whatsapp import business_account_list_params
from ....types.whatsapp.business_account_list_response import BusinessAccountListResponse
from ....types.whatsapp.business_account_retrieve_response import BusinessAccountRetrieveResponse

__all__ = ["BusinessAccountsResource", "AsyncBusinessAccountsResource"]


class BusinessAccountsResource(SyncAPIResource):
    """Manage Whatsapp business accounts"""

    @cached_property
    def phone_numbers(self) -> PhoneNumbersResource:
        return PhoneNumbersResource(self._client)

    @cached_property
    def settings(self) -> SettingsResource:
        """Manage Whatsapp business accounts"""
        return SettingsResource(self._client)

    @cached_property
    def with_raw_response(self) -> BusinessAccountsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return BusinessAccountsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BusinessAccountsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return BusinessAccountsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BusinessAccountRetrieveResponse:
        """
        Get a single Whatsapp Business Account

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/v2/whatsapp/business_accounts/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BusinessAccountRetrieveResponse,
        )

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[BusinessAccountListResponse]:
        """
        List Whatsapp Business Accounts

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v2/whatsapp/business_accounts",
            page=SyncDefaultFlatPagination[BusinessAccountListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    business_account_list_params.BusinessAccountListParams,
                ),
            ),
            model=BusinessAccountListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a Whatsapp Business Account

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/v2/whatsapp/business_accounts/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncBusinessAccountsResource(AsyncAPIResource):
    """Manage Whatsapp business accounts"""

    @cached_property
    def phone_numbers(self) -> AsyncPhoneNumbersResource:
        return AsyncPhoneNumbersResource(self._client)

    @cached_property
    def settings(self) -> AsyncSettingsResource:
        """Manage Whatsapp business accounts"""
        return AsyncSettingsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncBusinessAccountsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBusinessAccountsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBusinessAccountsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncBusinessAccountsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BusinessAccountRetrieveResponse:
        """
        Get a single Whatsapp Business Account

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/v2/whatsapp/business_accounts/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BusinessAccountRetrieveResponse,
        )

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[BusinessAccountListResponse, AsyncDefaultFlatPagination[BusinessAccountListResponse]]:
        """
        List Whatsapp Business Accounts

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v2/whatsapp/business_accounts",
            page=AsyncDefaultFlatPagination[BusinessAccountListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    business_account_list_params.BusinessAccountListParams,
                ),
            ),
            model=BusinessAccountListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a Whatsapp Business Account

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/v2/whatsapp/business_accounts/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class BusinessAccountsResourceWithRawResponse:
    def __init__(self, business_accounts: BusinessAccountsResource) -> None:
        self._business_accounts = business_accounts

        self.retrieve = to_raw_response_wrapper(
            business_accounts.retrieve,
        )
        self.list = to_raw_response_wrapper(
            business_accounts.list,
        )
        self.delete = to_raw_response_wrapper(
            business_accounts.delete,
        )

    @cached_property
    def phone_numbers(self) -> PhoneNumbersResourceWithRawResponse:
        return PhoneNumbersResourceWithRawResponse(self._business_accounts.phone_numbers)

    @cached_property
    def settings(self) -> SettingsResourceWithRawResponse:
        """Manage Whatsapp business accounts"""
        return SettingsResourceWithRawResponse(self._business_accounts.settings)


class AsyncBusinessAccountsResourceWithRawResponse:
    def __init__(self, business_accounts: AsyncBusinessAccountsResource) -> None:
        self._business_accounts = business_accounts

        self.retrieve = async_to_raw_response_wrapper(
            business_accounts.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            business_accounts.list,
        )
        self.delete = async_to_raw_response_wrapper(
            business_accounts.delete,
        )

    @cached_property
    def phone_numbers(self) -> AsyncPhoneNumbersResourceWithRawResponse:
        return AsyncPhoneNumbersResourceWithRawResponse(self._business_accounts.phone_numbers)

    @cached_property
    def settings(self) -> AsyncSettingsResourceWithRawResponse:
        """Manage Whatsapp business accounts"""
        return AsyncSettingsResourceWithRawResponse(self._business_accounts.settings)


class BusinessAccountsResourceWithStreamingResponse:
    def __init__(self, business_accounts: BusinessAccountsResource) -> None:
        self._business_accounts = business_accounts

        self.retrieve = to_streamed_response_wrapper(
            business_accounts.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            business_accounts.list,
        )
        self.delete = to_streamed_response_wrapper(
            business_accounts.delete,
        )

    @cached_property
    def phone_numbers(self) -> PhoneNumbersResourceWithStreamingResponse:
        return PhoneNumbersResourceWithStreamingResponse(self._business_accounts.phone_numbers)

    @cached_property
    def settings(self) -> SettingsResourceWithStreamingResponse:
        """Manage Whatsapp business accounts"""
        return SettingsResourceWithStreamingResponse(self._business_accounts.settings)


class AsyncBusinessAccountsResourceWithStreamingResponse:
    def __init__(self, business_accounts: AsyncBusinessAccountsResource) -> None:
        self._business_accounts = business_accounts

        self.retrieve = async_to_streamed_response_wrapper(
            business_accounts.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            business_accounts.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            business_accounts.delete,
        )

    @cached_property
    def phone_numbers(self) -> AsyncPhoneNumbersResourceWithStreamingResponse:
        return AsyncPhoneNumbersResourceWithStreamingResponse(self._business_accounts.phone_numbers)

    @cached_property
    def settings(self) -> AsyncSettingsResourceWithStreamingResponse:
        """Manage Whatsapp business accounts"""
        return AsyncSettingsResourceWithStreamingResponse(self._business_accounts.settings)
